'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _precedence;

var _csv = require('csv');

var _csv2 = _interopRequireDefault(_csv);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _lodash = require('lodash.max');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.fill');

var _lodash4 = _interopRequireDefault(_lodash3);

var _validator = require('validator');

var _validator2 = _interopRequireDefault(_validator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DATE = 'DATE';
var INTEGER = 'INTEGER';
var VARCHAR = 'VARCHAR';
var DOUBLE = 'DOUBLE PRECISION';
var EMPTY = 'EMPTY';

var precedence = (_precedence = {}, (0, _defineProperty3.default)(_precedence, VARCHAR, 0), (0, _defineProperty3.default)(_precedence, DATE, 1), (0, _defineProperty3.default)(_precedence, DOUBLE, 2), (0, _defineProperty3.default)(_precedence, INTEGER, 3), (0, _defineProperty3.default)(_precedence, EMPTY, 4), _precedence);

// Parses a csv and returns the data and SQL types

exports.default = function () {
  var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(csv, offset, limit) {
    var _ref, columns, rows;

    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (isNaN(offset)) {
              offset = 0;
            }if (isNaN(limit)) {
              limit = Infinity;
            }

            _context.next = 4;
            return parseCsv(csv, offset, limit);

          case 4:
            _ref = _context.sent;
            columns = _ref.columns;
            rows = _ref.rows;
            return _context.abrupt('return', {
              columns: columns,
              colTypes: inferTypes(rows, columns),
              data: rows
            });

          case 8:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));
  return function (_x, _x2, _x3) {
    return ref.apply(this, arguments);
  };
}();

// Infer the types of the csv


function inferTypes(csv, columns) {
  var colTypes = (0, _lodash4.default)(Array(csv[0].length), EMPTY);

  for (var col = 0; col < csv[0].length; ++col) {
    for (var row = 0; row < csv.length; ++row) {
      var value = csv[row][col];

      // Handle empty strings
      if (!value || value.match(/^$|^\s+$/)) {
        csv[row][col] = null;
        continue;
      } else if (value === null) {
        continue;
      }

      // Remove unnecessary symbols
      if (!_validator2.default.isEmail(value)) {
        csv[row][col] = value.replace(/[^\w\s\.\-\/]/g, '').trim();
      }

      // Determine the type of the value, and assign it if it has higher precedence
      var colType = determineType(csv[row][col]);
      if (precedence[colType] < precedence[colTypes[col]]) {
        colTypes[col] = colType;
      }
    }
  }

  // Remove empty columns and coltypes
  for (var i = colTypes.length; i > -1; --i) {
    if (colTypes[i] === VARCHAR) {
      colTypes[i] = 'VARCHAR(' + colMax(csv, i) + ')';
    } else if (colTypes[i] === EMPTY) {
      columns.splice(i, 1);
      colTypes.splice(i, 1);
      removeColumn(csv, i);
    }
  }

  return colTypes;
}

function removeColumn(matrix, col) {
  for (var row = 0; row < matrix.length; ++row) {
    matrix[row].splice(col, 1);
  }
}

function colMax(matrix, col) {
  return (0, _lodash2.default)(matrix.map(function (row) {
    return row[col] ? row[col].length : -1;
  }));
}

// Transform a CSV stream into JS arrays and objects
function parseCsv(csvStream, offset, limit) {
  return new _promise2.default(function (resolve) {
    var columns = void 0;
    var numRows = 0;
    var rows = [];

    // Stream the rows of the CSV into an array
    csvStream.pipe(_csv2.default.parse()).on('data', function (row) {
      if (numRows++ === offset) columns = row;else if (numRows > offset && numRows <= limit + 1 + offset) rows.push(row);
    }).on('finish', function () {
      return resolve({ columns: columns.map(sanitize), rows: rows });
    });
  });
}

// Make table and column names database friendly
function sanitize(str) {
  return str.toLowerCase() // make all characters lowercase
  .replace(/[\s\\\/\-\.:]+/g, '_') // replace certain characters with underscores
  .replace(/[^\w]/g, '') // remove the rest of the non alphanumeric characters
  .trim() // remove trailing and leading whitespace
  .replace(/_+$|^_+/g, '') // replace leading and trailing underscores
  .replace(/_+/g, '_'); // replace instances of multiple underscores with a single
}

// Determine the type of a string
function determineType(elem) {
  if (_validator2.default.isInt(elem)) {
    if (parseInt(elem, 10) > 2147483647) return DOUBLE;
    return INTEGER;
  } else if (_validator2.default.isFloat(elem)) {
    return DOUBLE;
  } else if (isDate(elem)) {
    return DATE;
  }
  return VARCHAR;
}

// Check if a string is a date
function isDate(input) {
  var formats = ['M/D/YYYY', 'DD/MM/YYYY', 'MM/DD/YYYY', 'YYYY-MM-DD'];
  return (0, _moment2.default)(input, formats, true).isValid();
}